# WARNING: Please don't edit this file. It was generated by Python/WinRT v2.0.0-beta.1

import enum

import winrt.system
from winrt import _winrt_windows_devices_lights

__all__ = [
    "LampArrayKind",
    "LampPurposes",
    "Lamp",
    "LampArray",
    "LampAvailabilityChangedEventArgs",
    "LampInfo",
]

class LampArrayKind(enum.IntEnum):
    UNDEFINED = 0
    KEYBOARD = 1
    MOUSE = 2
    GAME_CONTROLLER = 3
    PERIPHERAL = 4
    SCENE = 5
    NOTIFICATION = 6
    CHASSIS = 7
    WEARABLE = 8
    FURNITURE = 9
    ART = 10

class LampPurposes(enum.IntFlag):
    UNDEFINED = 0
    CONTROL = 0x1
    ACCENT = 0x2
    BRANDING = 0x4
    STATUS = 0x8
    ILLUMINATION = 0x10
    PRESENTATION = 0x20

Lamp = _winrt_windows_devices_lights.Lamp
LampArray = _winrt_windows_devices_lights.LampArray
LampAvailabilityChangedEventArgs = _winrt_windows_devices_lights.LampAvailabilityChangedEventArgs
LampInfo = _winrt_windows_devices_lights.LampInfo
